/*********************************************
This file includes all the PPI Functions
Modified by: S.M. Hasan, MPRG@Virginia Tech
*********************************************/


#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>

#include "ppi.h"
#include "ppiFM.h"

char	gPatternID;
UINT16	gPatternVal;
UINT16	gRows;
UINT16	gCols;
UINT16 *gImage = NULL;
int gImageSize;



CfgTable ppiMasterTable1D[] = {
	
	//{ CMD_PPI_PORT_ENABLE, CFG_PPI_PORT_ENABLE }, 	//this has been changed and tested for BF537 in 2005R4 RC2 kernel
 	{ CMD_PPI_PORT_DIRECTION, CFG_PPI_PORT_DIR_TX },
 	{ CMD_PPI_XFR_TYPE, CFG_PPI_XFR_TYPE_NO_SYNC },
 	{ CMD_PPI_PORT_CFG, CFG_PPI_PORT_CFG_SYNC1 },
 	{ CMD_PPI_FIELD_SELECT, CFG_PPI_FIELD_SELECT_1 },
 	{ CMD_PPI_PACKING, CFG_PPI_PACK_DISABLE }, //CFG_PPI_PACK_DISABLE
	{ CMD_PPI_SKIPPING, CFG_PPI_SKIP_DISABLE },
 	{ CMD_PPI_SKIP_ODDEVEN, CFG_PPI_SKIP_ODD },
 	{ CMD_PPI_DATALEN, CFG_PPI_DATALEN_10 }, //CFG_PPI_DATALEN_14
 	{ CMD_PPI_CLK_EDGE, CFG_PPI_CLK_EDGE_RISE },
 	{ CMD_PPI_TRIG_EDGE, CFG_PPI_TRIG_EDGE_RISE },
 	{ CMD_PPI_SET_DIMS, CFG_PPI_DIMS_1D },
 	{ CMD_PPI_DELAY, 0 },
 	{ CMD_PPI_SETGPIO },
 	//ioctl(fd, CMD_PPI_GET_ALLCONFIG);
	{ 0,0 }
},

ppiMasterTable2D[] = {
	{ CMD_PPI_PORT_DIRECTION, CFG_PPI_PORT_DIR_TX },  // write
	{ CMD_PPI_PACKING, CFG_PPI_PACK_DISABLE },
	{ CMD_PPI_SKIPPING, CFG_PPI_SKIP_DISABLE },
	{ CMD_PPI_SKIP_ODDEVEN, CFG_PPI_SKIP_ODD },
	{ CMD_PPI_DATALEN, CFG_PPI_DATALEN_16 },
	{ CMD_PPI_CLK_EDGE, CFG_PPI_CLK_EDGE_RISE },
	{ CMD_PPI_TRIG_EDGE, CFG_PPI_TRIG_EDGE_RISE },
	{ CMD_PPI_XFR_TYPE, CFG_PPI_XFR_TYPE_SYNC },
	{ CMD_PPI_PORT_CFG, CFG_PPI_PORT_CFG_SYNC23 }, 
	{ CMD_PPI_SET_DIMS, CFG_PPI_DIMS_2D },
	{ CMD_PPI_DELAY, 0 }, 
	{ CMD_PPI_SETGPIO },
	{ 0, 0 }
},

ppiSlaveTable1D[] = {
	
	//{ CMD_PPI_PORT_ENABLE, CFG_PPI_PORT_ENABLE }, 	// Read Data: this has been changed for BF537 in 2005R4 RC2 kernel
 	{ CMD_PPI_PORT_DIRECTION, CFG_PPI_PORT_DIR_RX },
 	{ CMD_PPI_XFR_TYPE, CFG_PPI_XFR_TYPE_NON646 },
 	{ CMD_PPI_PORT_CFG, CFG_PPI_PORT_CFG_NOSYNC },
 	{ CMD_PPI_FIELD_SELECT, CFG_PPI_FIELD_SELECT_IT },
 	{ CMD_PPI_PACKING, CFG_PPI_PACK_DISABLE }, //CFG_PPI_PACK_DISABLE
	{ CMD_PPI_SKIPPING, CFG_PPI_SKIP_DISABLE },
 	{ CMD_PPI_SKIP_ODDEVEN, CFG_PPI_SKIP_ODD },
 	{ CMD_PPI_DATALEN, CFG_PPI_DATALEN_16 }, //CFG_PPI_DATALEN_14
 	{ CMD_PPI_CLK_EDGE, CFG_PPI_CLK_EDGE_RISE },
 	{ CMD_PPI_TRIG_EDGE, CFG_PPI_TRIG_EDGE_RISE },
 	{ CMD_PPI_SET_DIMS, CFG_PPI_DIMS_1D },
 	{ CMD_PPI_DELAY, 1 },
 	{ CMD_PPI_SETGPIO },
 	//ioctl(fd, CMD_PPI_GET_ALLCONFIG);
	{ 0,0 }
},

ppiSlaveTable2D[] = {
	
	{ CMD_PPI_PORT_DIRECTION, CFG_PPI_PORT_DIR_RX },
 	{ CMD_PPI_XFR_TYPE, CFG_PPI_XFR_TYPE_NON646 },
 	{ CMD_PPI_PORT_CFG,CFG_PPI_PORT_CFG_XSYNC23 },// CFG_PPI_PORT_CFG_NOSYNC }
 	{ CMD_PPI_FIELD_SELECT,CFG_PPI_FIELD_SELECT_XT },// CFG_PPI_FIELD_SELECT_IT }
 	{ CMD_PPI_PACKING, CFG_PPI_PACK_DISABLE }, //CFG_PPI_PACK_DISABLE
	{ CMD_PPI_SKIPPING, CFG_PPI_SKIP_DISABLE },
 	//{ CMD_PPI_SKIP_ODDEVEN, CFG_PPI_SKIP_ODD },
 	{ CMD_PPI_DATALEN, CFG_PPI_DATALEN_16 }, //CFG_PPI_DATALEN_14
 	{ CMD_PPI_CLK_EDGE, CFG_PPI_CLK_EDGE_RISE },
 	{ CMD_PPI_TRIG_EDGE, CFG_PPI_TRIG_EDGE_RISE },
 	{ CMD_PPI_SET_DIMS, CFG_PPI_DIMS_2D },
 	{ CMD_PPI_DELAY, 0 },
 	{ CMD_PPI_SETGPIO },
 	//ioctl(fd, CMD_PPI_GET_ALLCONFIG);
	{ 0,0 }
};

/*
**	Function:	realloc_image()
**
**	Global Values Affected:
**  allocate image based on maximum gImageSize
**	reset gImageSize based on current values of rows & cols
*/

UINT16 *realloc_image()
{
	/* allocate image buffer */
	if (!gImage)
		gImage = (UINT16 *)malloc( (size_t)MAXIMAGESZ );

	if ( gImage == NULL ){
		perror("malloc");
	}

	gImageSize = (size_t)(gRows * gCols * sizeof(UINT16));

	printf("new gImageSize( %X ) = 0x%X (%d)\n", gImage, gImageSize, gImageSize );

	return gImage;
}


int fill_verifyBuffer( UINT16 *buf, UINT16 bufX, UINT16 bufY, 
				char pattern, UINT16 value, UINT8 fill )
{
	UINT16 i = 0;
	UINT16 x = 0;
	UINT16 y = 0;
	int		errorCnt = 0;

	if (fill)
		printf("fillBuffer(%X, %hd, %hd, '%c', 0x%hX)\n", 
				buf, bufX, bufY, pattern, value );

	while ( y < bufY ) {
		x = 0;
		while ( x < bufX ) {
			switch(pattern) {
				case 't':
				case 'T':
					/* tile of alternating row/column numbers */
					if ( x & 1 ){
						if (fill)
							*buf++ = x + 1;
						else
							if( *buf++ != ((x+1) & PPIVBITS) )
								errorCnt++;
					}
					else {
						if (fill)
							*buf++ = y + 1;
						else
							if( *buf++ != ((y+1) & PPIVBITS) )
								errorCnt++;
					}
					break;
				case 's':
				case 'S':
					/* use incremental counting pattern */
					if (fill)
						*buf++ = i++;
					else
						if( *buf++ != (i++ & PPIVBITS) )
							errorCnt++;
					break;
				case 'c':
				case 'C':
					/* use column number */
					if (fill)
						*buf++ = x + 1;	
					else
						if( *buf++ != ((x+1) & PPIVBITS) )
							errorCnt++;
					break;
				case 'r':
				case 'R':
					/* use row number */
					if (fill)
						*buf++ = y + 1;
					else
						if( *buf++ != ((y+1) & PPIVBITS) )
							errorCnt++;
					break;
				case 'v':
				case 'V':
				default:
					/* use supplied value */
					if (fill)
						*buf++ = value;
					else
						if( *buf++ != ( value & PPIVBITS) )
							errorCnt++;
					break;
			}
			x++;
		}
		y++;
	}
	return errorCnt;
}


void fillBuffer( UINT16 *buf, UINT16 bufX, UINT16 bufY, char pattern, UINT16 value )
{
	fill_verifyBuffer( buf, bufX, bufY, pattern, value, FILL_ONLY);
}


void showBuffer8( UINT8 *buf, UINT16 bufX, UINT16 bufY, UINT8 mask )
{
	UINT16 x,y = 0;

	while ( y < bufY ) {
		x = 0;
		while ( x < bufX ) {
			if ( x < 16 )  /* only show start of line */
				printf("%4hX ", (UINT16)(*buf & mask) );
			buf++;
			x++;
		}
		if( y < 8 )
			printf("\n");
		else{
			printf("\n....\n");
			break;	/* stop showing lines */
		}
		y++;
	}
}


void showBuffer( UINT16 *buf, UINT16 bufX, UINT16 bufY, UINT16 mask )
{
	UINT16 x,y = 0;

	while ( y < bufY ) {
		x = 0;
		while ( x < bufX ) {
			if ( x < 16 )  /* only show start of line */
				printf("%4hX ", *buf & mask);
			buf++;
			x++;
		}
		if( y < 8 )  /* only show first 8 lines */
			printf("\n");
		else{
			printf("\n....\n");
			break;	/* stop showing lines */
		}
		y++;
	}
}




int config_device( int devFD, CfgTable	*table )
{
	int	retval;

	int i = 0;
	// load up appropriate device configuration
	while ( table->cmd ){
		//	fgets( buf, sizeof(buf), stdin );
		retval = ioctl( devFD, table->cmd, table->arg );
		if(retval){
			perror("device ioctl error");
			return(-1);
		}
		//printf("Step %d (ioctl = %d, arg = %d)\n", i, table->cmd, table->arg );
		i++;
		table++;
	}
	return 0;
}

int beMaster(int ppiFD, UINT16 bufX, UINT16 bufY, int dims)
{
	int	retval;
	UINT16	value = 0;
	int done = 0;
	char	buf[128];
	CfgTable	*table;

	printf("Setting up master...");
#ifdef DEBUG
	ioctl( ppiFD, CMD_PPI_GET_ALLCONFIG, 0);
#endif

	gRows = bufX;
	gCols = bufY;
	gImage = realloc_image();
	if ( gImage == NULL ){
		return(-1);
	}

	/* fill in common values */
	if ( dims == DIM2D ){
		table = ppiMasterTable2D;
		/* 2D configuration */
		retval = ioctl( ppiFD, CMD_PPI_NUMLINES, bufY  );
		if(retval){
			perror("ppi ioctl error");
			return(-1);
		}
		retval = ioctl( ppiFD, CMD_PPI_LINELEN, bufX );
		if(retval){
			perror("ppi ioctl error");
			return(-1);
		}
	}
	else {
		/* 1D configuration */
		table = ppiMasterTable1D;
	}

	config_device( ppiFD, table );
	printf("Master set up complete...further input is transmited to slave\n");
#ifdef DEBUG
	ioctl( ppiFD, CMD_PPI_GET_ALLCONFIG, 0);
#endif

	while ( !done )
	{
		printf("Enter fill pattern [CcRrTtVvHh?]:  ");

		if ( fgets( buf, sizeof(buf), stdin ) == NULL ){
			done = 1;
			break;
		}
		switch( buf[0] ) {
			/*case 'q':
			case 'Q':
				done = 1;
				value = 0xdead;
				gPatternID = 'q';
				break; */

			case 'v':
			case 'V':
				gPatternID = 'v';
				printf("\nEnter fill value: ");
				if ( fgets( buf, sizeof(buf), stdin ) == NULL ){
					done = 1;
					break;
				}
				if ( sscanf(buf, "%hx", &value) != 1 ){
					printf("\nError reading value...retry\n");
					continue;
				}
				break;

			case 'h':
			case 'H':
			case '?':
				printf("\nColumn, Row, Sequential, Tile, Value, Help?, Quit\n");
				continue;
				break;

			default:
				gPatternID = buf[0];
		}
				
		fillBuffer( (UINT16 *)gImage, bufX, bufY, gPatternID, value );
		showBuffer( (UINT16 *)gImage, bufX, bufY, 0xFFFF );
		sleep(1);

		retval = write( ppiFD, gImage, gImageSize );
		if ( retval != gImageSize ){
			perror("ppi write error");
			done = 1;
		}
		
		done = 1;		// <<<<======= added for the loop-back test
	}

	printf("Done \n");

	return(0);
}


int beSlave(int ppiFD, UINT16 *IQ, UINT16 bufX, UINT16 bufY, int dims)
{
	int	retval, n;
	int	done = 0;
	int	bytes_read;
	static UINT16 x,y = 0;
	CfgTable	*table;
	static UINT16 mask, *buf;
	mask= 0xFFFF;
	
	gRows = bufX;
	gCols = bufY;
	//printf("Just before realloc_image()...\n");
	realloc_image();
	if ( gImage == NULL ){
		return(-1);
	}

	//printf("Setting up slave...to read %d bytes", gImageSize);


	if ( dims == DIM2D ){
		table = ppiSlaveTable2D;
		/* 2D configuration */
		retval = ioctl( ppiFD, CMD_PPI_NUMLINES, bufY  );
		if(retval){
			perror("ppi ioctl error");
			return(-1);
		}
		retval = ioctl( ppiFD, CMD_PPI_LINELEN, bufX );
		if(retval){
			perror("ppi ioctl error");
			return(-1);
		}
	}
	else {
		/* 1D configuration */
		table = ppiSlaveTable1D;
		retval = ioctl( ppiFD, CMD_PPI_XFR_TYPE, CFG_PPI_XFR_TYPE_NO_SYNC );
		if(retval){
			perror("ppi ioctl error");
			return(-1);
		}
		retval = ioctl( ppiFD, CMD_PPI_PORT_CFG, CFG_PPI_PORT_CFG_SYNC1 );
		if(retval){
			perror("ppi ioctl error");
			return(-1);
		}
	}
	config_device( ppiFD, table );

	/* show configuration */
	ioctl( ppiFD, CMD_PPI_GET_ALLCONFIG, 0);

	//printf("Slave set up complete...following output is received from master...\n");

	// read image from PPI 
	// print out what is received

	//while (!done) {
		bytes_read = read( ppiFD, (char *)gImage, gImageSize );
		if ( (*gImage & 0x07FF) == 0x06ad )
			done = 1;
		else{
			//printf("Printing Buffer values ...........\n");
			//showBuffer( (UINT16 *)gImage, bufX, bufY, 0xFFFF );	// showing the whole buffer
			processIQ( (UINT16 *)gImage, IQ, bufX, bufY, 0xFFFF);
		}

		//sleep(1);
	//}

	return(0);

}

void processIQ( UINT16 *buf, UINT16 *IQ, UINT16 bufX, UINT16 bufY, UINT16 mask)
{
	

       		UINT16 x,y = 0;
		while ( y < bufY ) {
			x = 0;
			while ( x < bufX ) {
				if ( x < 16 )  /* only show start of line */
					*IQ= *(buf) & mask;
					buf++;
					x++;
					IQ++;
			}
		if( y >= NUMLINES )  /* only show first 8 lines */
			return;	/* stop showing lines */
			y++;
		}
	
	
}

void showUsage(char *pgmname)
{
		printf("Usage: %s [m-master | s-slave | l-loop ]\n", pgmname);
}

void closePPI(int ppiFD)
{
int	retval;

if (gImage)
		free(gImage);

	retval = close( ppiFD );
	if(retval)
		perror("ppi close error");

}
	


