module AD2DA_FIR_NCO (
	// PLL ports
	rst_b,
	clk_osc14p,	// 80MHz from OSC to PLL5.
	clk_osc4p,	// 80MHz from OSC to PLL6.
	ADclk,		// 120MHz to AD
	DA1clk,		// 240kHz to DA1
	DA2clk,		// 240kHz to DA2

	// A/D data
	inAD,
	// D/A data
	outDA1,
	outDA2,
	// PPI interface
	ppi_clk,
	ppi_data,
	frame,
	line,
	// Monitoring
	p_J22,
	p_J24
	);
// Clock management
input		rst_b;			// active low reset
// 80MHz clock from OSC.
input		clk_osc14p;
input		clk_osc4p;
output		ADclk;
output		DA1clk,DA2clk;	// 240kHz clock outputs to A/D and D/A
// AD interface
input	[11:0]	inAD;		// two's complement form
// DA interface
output	[13:0]	outDA1;		// unsigned binary
output	[13:0]	outDA2;		// unsigned binary
// PPI interface
output		ppi_clk;
output	[15:0]	ppi_data;
output		frame,line;
// Monitoring
output		p_J24;
output		p_J22;

 wire		rst_h,p5clk,p6clk;
 wire	[11:0]	fsin_o,fcos_o;	//2's comp
 //unsigned
 wire	[11:0]	nsi_inAD;
 wire	[11:0]	nsi_fcos_o;
 wire	[11:0]	nsi_fsin_o;
 wire	[11:0]	ns_dwnADI,ns_dwnADQ;
 reg	[11:0]	dwnADI,dwnADQ;
 wire	[11:0]	flt_dwnADI,flt_dwnADQ;
 wire	[11:0]	outDA1_12,outDA2_12;
 wire	[11:0]	ns_toutDA1_12,ns_toutDA2_12;
 reg	[11:0]	toutDA1_12,toutDA2_12;
 reg	[13:0]	outDA1,outDA2;
 reg		ns_flag,flag;
 reg	[15:0]	ppi_data0, ns_ppi_data0;
 reg	[15:0]	ppi_data1;

 reg	[4:0]	dec_counter, ns_dec_counter;

 wire		done1,done2;
 wire		clk15MHz,clk30MHz;
 wire		ppi_clk;
 assign	ppi_clk=clk30MHz;
 wire		dec_flag;
 assign	dec_flag=(dec_counter[4:1]==4'b000);

 assign	rst_h=~rst_b;
 assign	p_J24=clk15MHz;
 assign	p_J22=dec_flag;

// PLL's
 myPLL5	inst_myPLL5 (
	 	     .inclk0(clk_osc14p),
		     .pllena(1'b1),
		     .areset(rst_h),
		     .c0(p5clk),	// 120MHz to NCO
		     .e0(ADclk));	// 120MHz to ADC
 myPLL6	inst_myPLL6 (
	 	     .inclk0(clk_osc4p),
		     .pllena(1'b1),
		     .areset(rst_h),
		     .c0(p6clk),
		     .c1(clk15MHz),
		     .c2(clk30MHz),
		     .e0(DA1clk),
		     .e1(DA2clk));
// NCO 
 myNCO	inst_myNCO (
	 	    .phi_inc_i(8_9478_4853),
		    .clk(p5clk),	// 120MHz from PLL5
		    .reset(rst_h),
		    .clken(1'b1),
		    .fsin_o(fsin_o),
		    .fcos_o(fcos_o),
		    .data_ready(dummy0));
// 2's comp to unsigned value
 assign	nsi_inAD=(inAD[11])?{1'b0,inAD[10:0]}:{1'b1,inAD[10:0]};
 assign	nsi_fcos_o=(fcos_o[11])?{1'b0,fcos_o[10:0]}:{1'b1,fcos_o[10:0]};
 assign	nsi_fsin_o=(fsin_o[11])?{1'b0,fsin_o[10:0]}:{1'b1,fsin_o[10:0]};
// Unsigned multiplication
 myMulti inst_myMultiI (
	 		.dataa(nsi_fcos_o),
			.datab(nsi_inAD),
			.result(ns_dwnADI));
 myMulti inst_myMultiQ (
	 		.dataa(nsi_fsin_o),
			.datab(nsi_inAD),
			.result(ns_dwnADQ));

 always @(posedge p5clk or negedge rst_b)
 begin
	 if (~rst_b)
	 begin
		 dwnADI<=3'h000;
		 dwnADQ<=3'h000;
	 end
	 else
	 begin
		 dwnADI<=ns_dwnADI;
		 dwnADQ<=ns_dwnADQ;
	 end
 end
// FIR-decimation
 myFIR	inst_myFIRI (
		     .clk(p5clk),		// 120MHz
		     .rst(rst_h),
		     .data_in(dwnADI),
		     .fir_result(flt_dwnADI),
		     .done(done1),
		     .rdy_to_ld(dummy1));
 myFIR	inst_myFIRQ (
		     .clk(p5clk),		// 120MHz
		     .rst(rst_h),
		     .data_in(dwnADQ),
		     .fir_result(flt_dwnADQ),
		     .done(done2),
		     .rdy_to_ld(dummy3));
 FIFO_DA inst_FIFO_DAI (
	 		.rst_b(rst_b),
			.wrclk(p5clk),		// 120MHz
			.data(flt_dwnADI),	// 12-bit
			.done(done1),		// from FIR output
			.rdclk(p6clk),		// 7.5MHz
			.odata(ns_toutDA1_12)	// 12-bit
				);
 FIFO_DA inst_FIFO_DAQ (
	 		.rst_b(rst_b),
			.wrclk(p5clk),		// 120MHz
			.data(flt_dwnADQ),	// 12-bit
			.done(done2),		// from FIR output
			.rdclk(p6clk),		// 7.5MHz
			.odata(ns_toutDA2_12)	// 12-bit
				);
//7.5MHz clock(p6clk)
 always @(posedge p6clk or negedge rst_b)
 begin
	 if (~rst_b)
	 begin
		 outDA1<=4'h0000;
		 outDA2<=4'h0000;
		 toutDA1_12<=12'b0000_0000_0000;
		 toutDA2_12<=12'b0000_0000_0000;
	 end
	 else
	 begin
		 outDA1<={ns_toutDA1_12,2'b00};
		 outDA2<={ns_toutDA2_12,2'b00};
		 toutDA1_12<=ns_toutDA1_12;
		 toutDA2_12<=ns_toutDA2_12;
	 end
 end

 // Filtering for additional decimation
FIR_lowf I_FIR_lowf (
		.clk(p6clk),
		.rst(rst_h),
		.data_in(toutDA1_12),
		.fir_result(outDA1_12),
		.done(dummy4),
		.rdy_to_ld(dummy5));
 
FIR_lowf Q_FIR_lowf (
		.clk(p6clk),
		.rst(rst_h),
		.data_in(toutDA2_12),
		.fir_result(outDA2_12),
		.done(dummy6),
		.rdy_to_ld(dummy7));

 // counter with 15MHz to generate the FIFO write_enable signal
 always @(dec_counter)
 begin
	 ns_dec_counter=dec_counter+1'b1;
 end
 // clk15MHz is 15MHz
 always @(negedge clk15MHz or negedge rst_b)
 begin
	 if (~rst_b)
	 begin
		 flag<=1'b0;
		 ppi_data0<=16'h0000;
		 dec_counter<=5'b0_0000;
	 end
	 else
	 begin
		 flag<=ns_flag;
		 ppi_data0<=ns_ppi_data0;
		 dec_counter<=ns_dec_counter;
	 end
 end
 // data is update with 2*7.5MHz
 always @(flag or outDA1_12 or outDA2_12 or ppi_data0)
 begin
	 ns_flag=flag+1'b1;
	 ns_ppi_data0=ppi_data0;
	 if (flag)
	 begin
		 ns_ppi_data0={outDA1_12,4'h0};
	 end
	 else
	 begin
		 ns_ppi_data0={outDA2_12,4'h0};
	 end
 end

 // transfer negedge data to posedge data
 always @(posedge clk15MHz or negedge rst_b)
 begin
	 if (~rst_b)
	 begin
		 ppi_data1<=16'h0000;
	 end
	 else
	 begin
		 ppi_data1<=ppi_data0;
	 end
 end

FIFO_FIFOcontrol_FSM inst_FIFO_FIFOcontrol_FSM (
						// input
						.clk15MHz(clk15MHz),
						.dec_flag(dec_flag),
						.clk30MHz(clk30MHz),
						.rst_b(rst_b),
				
						.in_data(ppi_data1),	// 16bit data
						// output
						.frame(frame),
						.line(line),
						.ppi_data(ppi_data)	// ppi_data
						);
endmodule
