module FIFO_FIFOcontrol_FSM (
		// input
		clk15MHz,
		dec_flag,
		clk30MHz,
		rst_b,

		in_data,	// 16bit data
		// output
		frame,
		line,
		ppi_data	// ppi_data
		);
// input
input		clk15MHz;
input		dec_flag;
input		clk30MHz;
input		rst_b;
input	[15:0]	in_data;	// 16bit data
		// output
output		frame;
output		line;
output	[15:0]	ppi_data;	// ppi_data

parameter	IDLE=1'b0, TX_MODE=1'b1;
reg		cs_fsm, ns_fsm;
reg		hit_tx, ns_hit_tx;
reg	[15:0]	ppi_data;
wire	[15:0]	ns_ppi_data;
wire		fifo_rdy,tx_done;
wire	[8:0]	rdusedw;
assign	fifo_rdy=(rdusedw==255);

myFIFO2 inst_myFIFO2 (
			.data(in_data),
			//.wrreq(1'b1),
			.wrreq(dec_flag),
			.rdreq(frame&line),
			.rdclk(~clk30MHz),
			.wrclk(clk15MHz),
			.q(ns_ppi_data),
			.rdusedw(rdusedw));

// Fifo control FSM
always @(cs_fsm or fifo_rdy or hit_tx or tx_done)
begin
	ns_fsm=cs_fsm;
	ns_hit_tx=hit_tx;
	case (cs_fsm)
		IDLE:
		begin
			if (fifo_rdy)
			begin
				ns_fsm=TX_MODE;
				ns_hit_tx=1'b1;
			end
		end
		TX_MODE:
		begin
			if (tx_done)
			begin
				ns_fsm=IDLE;
				ns_hit_tx=1'b0;
			end
		end
	endcase
end

always @(posedge clk30MHz or negedge rst_b)
begin
	if (~rst_b)
	begin
		cs_fsm<=1'b0;
		hit_tx<=1'b0;
	end
	else
	begin
		cs_fsm<=ns_fsm;
		hit_tx<=ns_hit_tx;
	end
end

always @(negedge clk30MHz or negedge rst_b)
begin
	if (~rst_b)
	begin
		ppi_data<=16'h0000;
	end
	else
	begin
		ppi_data<=ns_ppi_data;
	end
end

// Fifo output controller
C271 inst_C271 (
		// input
		.clk(clk30MHz),		// must be 30MHz
		.cnt_en(hit_tx),
		.rst_b(rst_b),
		// output
		.frame(frame),
		.line(line),
		.tx_done(tx_done)
		);

endmodule
